"""Common CLI commands stuff"""

import logging
import sys

verbosity_to_log_level = {
    0: logging.CRITICAL,
    1: logging.WARNING,
    2: logging.INFO,
    3: logging.DEBUG,
}


# pylint: disable=too-few-public-methods
class StdoutInfoFilter(logging.Filter):
    """Logging filter to keep messages only >= logging.INFO"""

    def filter(self, record):
        return record.levelno in (logging.DEBUG, logging.INFO)


def console_log(logger_name=None):
    """Decorator to initialize CLI command console logging according to specified verbosity"""

    def decorator(function):
        def decorated_function(*args, **kwargs):
            # Add StreamHandler for console output
            root_logger = logging.getLogger()
            log_level = verbosity_to_log_level[kwargs["verbosity"]]
            formatter = logging.Formatter(fmt="%(levelname)s - %(message)s")

            # ensure log messages are displayed only once on terminal
            stream_handlers = [
                x
                for x in root_logger.handlers
                if isinstance(x, logging.StreamHandler)
                if x.stream.isatty()
            ]
            if stream_handlers:
                for handler in stream_handlers:
                    handler.setFormatter(formatter)
                    handler.setLevel(level=log_level)
            else:
                if log_level < logging.WARNING:
                    stdout_console_handler = logging.StreamHandler(sys.stdout)
                    stdout_console_handler.addFilter(StdoutInfoFilter())
                    stdout_console_handler.setLevel(log_level)
                    stdout_console_handler.setFormatter(formatter)
                    root_logger.addHandler(stdout_console_handler)

                stderr_console_handler = logging.StreamHandler(sys.stderr)
                stderr_console_handler.setLevel(
                    log_level if log_level > logging.WARNING else logging.WARNING
                )
                stderr_console_handler.setFormatter(formatter)
                root_logger.addHandler(stderr_console_handler)

            # Set logger log level
            logging.getLogger(logger_name).setLevel(log_level)

            return function(*args, **kwargs)

        return decorated_function

    return decorator
