"""Initialize OIDC IDP configuration command"""

import base64
import hashlib
import json
import logging
import os.path

from django.core.management.base import BaseCommand
from jwcrypto import jwk

from eeauthentic2.management.commands import console_log

DEFAULT_OUTPUT = "-"
DEFAULT_KEY_TYPE = "RSA"
DEFAULT_KEY_LENGTH = 4096
DEFAULT_KEY_USE = "sig"
DEFAULT_KEY_ALGO = "RS256"

log = logging.getLogger(__name__)


class Command(BaseCommand):
    """Command to initialize OIDC IDP configuration"""

    def add_arguments(self, parser):
        """Add command argument to parser"""
        parser.add_argument(
            "-o",
            "--output",
            help=f"OIDC configuration output file (use '-' to stdout, default: {DEFAULT_OUTPUT})",
            default=DEFAULT_OUTPUT,
        )

        parser.add_argument(
            "-t",
            "--key-type",
            help=f"OIDC JWK key type (default: {DEFAULT_KEY_TYPE})",
            default=DEFAULT_KEY_TYPE,
        )

        parser.add_argument(
            "-l",
            "--key-length",
            type=int,
            help=f"OIDC JWK key length (default: {DEFAULT_KEY_LENGTH})",
            default=DEFAULT_KEY_LENGTH,
        )

        parser.add_argument(
            "-u",
            "--key-use",
            help=f"OIDC JWK key use (default: {DEFAULT_KEY_USE})",
            default=DEFAULT_KEY_USE,
        )

        parser.add_argument(
            "-a",
            "--key-algo",
            help=f"OIDC JWK key algorithm (default: {DEFAULT_KEY_ALGO})",
            default=DEFAULT_KEY_ALGO,
        )

    # pylint: disable=unused-argument
    @console_log(__name__)
    def handle(
        self,
        *args,
        output=None,
        key_type=None,
        key_length=None,
        key_use=None,
        key_algo=None,
        **kwargs,
    ):
        """Command entrypoint"""
        if output != "-" and os.path.exists(output):
            log.fatal("Output file '%s' already exists.", output)
            return False

        log.info("Initialize Authentic2 OIDC IDP support in %s", output)

        log.info(
            "Generate a %s key (length: %d, use: %s, algorithm: %s)...",
            key_type,
            key_length,
            key_use,
            key_algo,
        )
        k = jwk.JWK.generate(kty=key_type, size=key_length, use=key_use, alg=key_algo)
        log.info("Done.")

        log.info("Compute KID...")
        kid = (
            base64.urlsafe_b64encode(
                hashlib.sha256(k.export_to_pem(private_key=False, password=None)).digest()
            )
            .decode("utf-8")
            .rstrip("=")
        )
        # Note: Starting 0.9 version of jwcrypto, JWK is a dict-like object, but before,
        # to set kid, we need to set it in its _params property.
        if isinstance(k, dict):
            k["kid"] = kid
        else:
            k._params["kid"] = kid  # pylint: disable=protected-access

        s = jwk.JWKSet()
        s["keys"].add(k)

        oidc_configuration = (
            "# vim: shiftwidth=4 tabstop=4 expandtab\n\n"
            f"A2_IDP_OIDC_JWKSET = {json.dumps(s.export(as_dict=True), indent=4)}"
        )

        if output == "-":
            log.info("Export JWK set to stdout...")
            return oidc_configuration

        log.info("Export JWK set to %s...", output)
        with open(output, "w", encoding="utf8") as fd:
            fd.write("# vim: shiftwidth=4 tabstop=4 expandtab\n\n")
            fd.write(f"A2_IDP_OIDC_JWKSET = {json.dumps(s.export(as_dict=True), indent=4)}")
        log.info("Done.")

        return f"Authentic2 OIDC IDP configuration saved in {output}."
