"""
Reload command
Reload Authentic2 service
"""

import logging
import subprocess  # nosec: B404

from django.core.management.base import BaseCommand

from eeauthentic2.management.commands import console_log

log = logging.getLogger(__name__)

DEFAULT_PERIOD_LENGTH = 300


def reload_authentic2(just_try=False):
    """Reload authentic2 service"""
    # To reload authentic2 service, we need its main PID
    try:
        log.info("Get authentic2 service main PID")
        main_pid = subprocess.check_output(
            ["/usr/bin/systemctl", "show", "--property", "MainPID", "--value", "authentic2"],
        ).strip()
    except subprocess.CalledProcessError as err:
        log.error(
            "An error occurred retrieving Authentic2 service main PID (exit code: %d):\n%s",
            err.returncode,
            err.output,
        )
        return False

    try:
        main_pid = int(main_pid)
    except ValueError:
        log.error(
            "Invalid value retrieved for Authentic2 service main PID (%s), "
            "can't reload its configuration",
            main_pid,
        )
        return False

    if main_pid == 0:
        log.info("Authentic2 service is not running, do not reload its configuration")
        return True
    log.info("Authentic2 service main PID: %d", main_pid)

    # Reload authentic2 configuration by sending a SIGHUP to service main PID
    try:
        log.info("Reload authentic2 configuration")
        if not just_try:
            subprocess.run(
                ["/usr/bin/kill", "-sHUP", str(main_pid)],
                capture_output=True,
                check=True,
            )
        else:
            log.debug("Just-try mode, not really reload configuration.")
    except subprocess.CalledProcessError as err:
        log.error(
            "An error occurred reloading Authentic2 configuration (exit code: %d):\n%s",
            err.returncode,
            err.output,
        )
        return False
    return True


class Command(BaseCommand):
    """
    Authentic2 metrics command
    """

    def add_arguments(self, parser):
        """Add command argument to parser"""
        parser.add_argument("-j", "--just-try", action="store_true", help="Enable just-try mode")

    @console_log(__name__)
    def handle(self, *args, **kwargs):  # pylint: disable=unused-argument
        """Command entrypoint"""
        return 0 if reload_authentic2(kwargs["just_try"]) else 1


# vim: tabstop=4 shiftwidth=4 softtabstop=4 expandtab
