"""Passwords addons for Authentic2"""

import json
import logging
import os.path

import zxcvbn
from authentic2.passwords import PasswordChecker
from django.conf import settings
from django.core.exceptions import ValidationError
from django.http import JsonResponse
from zxcvbn.matching import add_frequency_lists

from .gettext import _, get_translations

log = logging.getLogger(__name__)


def validate_password(get_response):
    """
    Validate password middleware for Django

    This middleware replace Authentic2 /api/validate-password/ route handler by a custom one.

    Note: This middleware is supposed to be used with :
      - the prodived authentic2/widgets/new_password.html template file
      - the prodived authentic2/js/password.js static file
    """

    def middleware(request):
        """Handle a request"""
        if request.path == "/api/validate-password/":
            data = json.loads(request.body)
            checker = ZxcvbnPasswordChecker()
            result = checker.check_password(data["password"])
            return JsonResponse(result)
        return get_response(request)

    return middleware


class ZxcvbnPasswordChecker(PasswordChecker):
    """
    Password checker based on Zxcvbn lib
    """

    def __init__(self):
        if self.custom_dictionaries_file:
            log.info("Load Zxcvbn custom dictionaries from %s", self.custom_dictionaries_file)
            add_frequency_lists(self.custom_dictionaries)

    @property
    def min_score(self):
        """Minimum Zxcvbn score to approved a password"""
        return (
            settings.PASSWORD_ZXCVBN_MIN_SCORE
            if hasattr(settings, "PASSWORD_ZXCVBN_MIN_SCORE")
            else 4
        )

    @property
    def min_guesses_log10(self):
        """Minimum Zxcvbn guesses_log10 to approved a password"""
        return (
            settings.PASSWORD_ZXCVBN_MIN_GUESSES_LOG10
            if hasattr(settings, "PASSWORD_ZXCVBN_MIN_GUESSES_LOG10")
            else 12
        )

    @property
    def ban_dictionaries(self):
        """Banned dictionaries list"""
        return (
            settings.PASSWORD_ZXCVBN_BANNED_DICTIONARIES
            if hasattr(settings, "PASSWORD_ZXCVBN_BANNED_DICTIONARIES")
            else []
        )

    @property
    def show_warnings(self):
        """Show Zxcvbn warnings"""
        return (
            settings.PASSWORD_ZXCVBN_SHOW_WARNINGS
            if hasattr(settings, "PASSWORD_ZXCVBN_SHOW_WARNINGS")
            else True
        )

    @property
    def show_suggestions(self):
        """Show Zxcvbn suggestions"""
        return (
            settings.PASSWORD_ZXCVBN_SHOW_SUGGESTIONS
            if hasattr(settings, "PASSWORD_ZXCVBN_SHOW_SUGGESTIONS")
            else True
        )

    @property
    def custom_dictionaries_file(self):
        """Return custom dictionaries JSON file path to use (if configured)"""
        return (
            settings.PASSWORD_ZXCVBN_CUSTOM_DICTIONARIES
            if hasattr(settings, "PASSWORD_ZXCVBN_CUSTOM_DICTIONARIES")
            else None
        )

    @property
    def custom_dictionaries(self):
        """Load custom dictionaries JSON file (if configured)"""
        if not self.custom_dictionaries_file:
            return {}
        with open(self.custom_dictionaries_file, encoding="utf8") as fd:
            return json.load(fd)

    _zxcvbn_translator = None

    @property
    def zxcvbn_translator(self):
        """Return GNUTranslations object for Zxcvbn domain"""
        if self._zxcvbn_translator is None:
            self._zxcvbn_translator = get_translations(
                "zxcvbn",
                localedir=os.path.join(os.path.abspath(os.path.dirname(zxcvbn.__file__)), "locale"),
            )
        return self._zxcvbn_translator.gettext

    def __call__(self, password, **kwargs):
        """
        Password check method call by Authentic 2 to check a password

        This method normally return a list of checks done, each check represent by an
        instance of the authentic2.passwords.PasswordChecker.Check class. We do not use this
        method to control the error display to the user. Instead, we directly raise ValidationError
        in case of error. When no error occurred validating the provided password, just return an
        empty list.

        Note:
          Authentic2 run the password checker a first time with an empty password to list checks,
          just return an empty list in this case.
        """
        if password:
            result = self.check_password(password)
            if result["errors"]:
                raise ValidationError(
                    " ".join(
                        [_("This password is not accepted.")]
                        + result["errors"]
                        + result["suggestions"]
                    )
                )
        return []

    def check_password(self, password):
        """
        The real password check method

        :returns: {
            "score": [int],
            "guesses_log10": [int],
            "errors": [ [error messages] ],
            "suggestions": [ [suggestion messages] ],
        }
        """
        # Zxcvbn does not support empty password, handle the specified case manually.
        if not password:
            return {
                "score": 0,
                "guesses_log10": 0,
                "errors": [
                    _("Empty password are forbidden."),
                ],
                "suggestions": [],
            }
        results = zxcvbn.zxcvbn(password)

        log.debug("Zxcvbn result: %s", results)
        log.info("Zxcvbn score: %s", results["score"])
        log.info("Zxcvbn guesses log10: %s", results["guesses_log10"])
        log.info("Zxcvbn warning: %s", results["feedback"]["warning"])
        log.info("Zxcvbn suggestions: %s", results["feedback"]["suggestions"])

        banned_tokens = []
        if self.ban_dictionaries:
            for match in results["sequence"]:
                if (
                    match["pattern"] == "dictionary"
                    and match["dictionary_name"] in self.ban_dictionaries
                ):
                    banned_tokens.append(match["token"])
            log.debug("Zxcvbn banned token(s) found: '%s'", "', '".join(banned_tokens))

        result = {
            "score": results["score"],
            "guesses_log10": results["guesses_log10"],
            "errors": [],
            "suggestions": (
                [self.zxcvbn_translator(msg) for msg in results["feedback"]["suggestions"] if msg]
                if results["feedback"]["suggestions"] and self.show_suggestions
                else []
            ),
        }

        if (
            (not self.min_score or results["score"] >= self.min_score)
            and (not self.min_guesses_log10 or results["guesses_log10"] >= self.min_guesses_log10)
            and not banned_tokens
        ):
            return result

        if results["feedback"]["warning"] and self.show_warnings:
            result["errors"].append(self.zxcvbn_translator(results["feedback"]["warning"]))
        if banned_tokens:
            result["errors"].append(_("Banned token found in this password."))
        if not result["errors"]:
            result["errors"].append(_("The security of this password is too weak."))

        return result


# vim: tabstop=4 shiftwidth=4 softtabstop=4 expandtab
